#ifndef WEBWATCH_WATCHVIEW_H
#define WEBWATCH_WATCHVIEW_H

// ***************************************************************************
// System Includes
// ***************************************************************************

#include <View.h>

// ***************************************************************************
// Project Includes
// ***************************************************************************

// Standard type and constant definitions.
#include "StyleGuideDefs.h"

// ***************************************************************************
// External Name Declarations
// ***************************************************************************

// ***************************************************************************
// External Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Constant Definitions
// ***************************************************************************

// ***************************************************************************
// Type Definitions
// ***************************************************************************

// ***************************************************************************
// Class Definition
// ***************************************************************************

/**
 * The WatchView class forms the heart of the WebWatch replicant. 
 *
 * When instantiated from the Deskbar using the Instantiate() function, this
 * class creates a small view to display the current Internet Time, measured
 * in beats (each of which is 86.4 seconds) relative to Biel Mean Time (BMT)
 * in Switzerland. 
 */
class WatchView : public BView 
{
public:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

	/**
	 * Default constructor. 
	 *
	 * This constructor is used by the WatchApp class to install
	 * the WebWatch replicant into the Deskbar.
	 */
	WatchView();

	/**
	 * Creates a new WatchView object from a BMessage. 
	 *
	 * This constructor is used to "rehydrate" the archived WatchView object
	 * after the Deskbar (or any other BShelf) has received it. It is called
	 * by Instantiate().
	 */
	WatchView(BMessage* message);
	
	/** Automatically generated copy constructor. */
	// WatchView(const WatchView& old);

	/** Automatically generated destructor. */
	// virtual ~WatchView();

	/** Automatically generated assignment operator. */
	// WatchView& operator=(WatchView& right);

	/** 
	 * Creates a new WatchView object from an archived BMessage.
	 *
	 * This function is called by the Deskbar's shelf after it has received
	 * the message that contains our replicant. (To enable the Deskbar to 
	 * find this function, we export it from the executable.) 
	 */
	static WatchView* Instantiate(BMessage* archive);

	/** 
	 * Creates an archive of the data that we need in order to instantiate 
	 * ourselves.
	 */
	virtual status_t Archive(BMessage* archive, bool deep = true) const;

	/**
	 * Draws the view.
	 */
	virtual void Draw(BRect updateRect);

	/**
	 * Creates the pop-up menu on-the-fly when the mouse is clicked inside 
	 * the view, and waits for the user to make a selection.
	 */
	virtual void MouseDown(BPoint point);

	/**
	 * Handles the messages that are sent by the items from the pop-up menu.
	 */
	virtual void MessageReceived(BMessage* message);

	/**
	 * Periodically checks whether the Internet Time has changed, and tells
	 * the view to redraw itself. The time between these checks depends on
	 * the pulse rate of the Deskbar window. By default the pulse rate is 500
	 * milliseconds, which is fine for our purposes. Note that other Deskbar
	 * replicants could possibly change the pulse rate of the Deskbar window.
	 */
	virtual void Pulse(void);

protected:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	// Attribute Definitions =================================================

	// Member Function Definitions ===========================================

private:
	// Constant Definitions ==================================================

	// Type Definitions ======================================================

	/** The name of our superclass. */
	typedef BView super;
	
	// Attribute Definitions =================================================

	/** The Internet Time from the last call to Pulse(). */
	uint32 oldInternetTime;

	// Member Function Definitions ===========================================

	/** Returns the current Internet Time. */
	int32 GetInternetTime(void);
};

// ***************************************************************************
// Global Variable Declarations
// ***************************************************************************

// ***************************************************************************
// Inline Function Definitions
// ***************************************************************************

// ***************************************************************************
// Helper Function Declarations
// ***************************************************************************

// ***************************************************************************
// Global Function Declarations
// ***************************************************************************

#endif // WEBWATCH_WATCHVIEW_H
